#include <iostream>
#include <iomanip>
#include <math.h>
#define NMAX 30
using namespace std;
//
// class for numerical integration by the Gauss-Legendre rule
//
class gauss_legendre_rule {
  int nmax;  // the maximal number of the nodes of the Gauss-Legendre rule
  double xi[NMAX+1][NMAX+1]; //   the nodes x(n,i) (i=1,...,n) of the n-point rule
  double wi[NMAX+1][NMAX+1]; // the weights w(n,i) (i=1,...,n) of the n-point rule
public:
  gauss_legendre_rule(int nmax0);
  double integral(double (*func)(double), double xmin, double xmax, int n);
private:
  double pol(int n, double x), dpol(int n, double x);
  void get_node(void);
  void get_weight(void);
};
//
// constructor of the class "gauss_legendre_rule"
//
gauss_legendre_rule::gauss_legendre_rule(int nmax0)
{
  cout << setiosflags(ios::scientific);
  //
  if (nmax0>NMAX)
	{
	  cout << "nmax must satisfy nmax <= " << setw(3) << NMAX << endl;
	  nmax = NMAX;
	}
  if (nmax0<2)
	{
	  cout << "nmax must satisfy nmax >= 2 " << endl;
	  nmax = 2;
	}
  if (nmax0>=2 && nmax0<=NMAX)
	nmax = nmax0;
  //
  cout << "# the maximal number of nodes : nmax = " << nmax << endl;
  //
  get_node();
  get_weight();
}
//
// numerical integration of f(x) over a finite interval (xmin, xmax)
// by the Gauss-Legendre rule
//
// func : the integrand
// xmin, xmax : the end points of the integral interval (xmin, xmax)
// n : the number of nodes
//
double gauss_legendre_rule::integral(double (*func)(double), double xmin, double xmax, int n)
{
  double s = 0.0;
  //
  if (xmax <= xmin)
	cout << "xmin and xmax must satisfy xmin < xmax" << endl;
  else
	{
	  double c1, c2;
	  c1 = 0.5 * (xmax + xmin);
	  c2 = 0.5 * (xmax - xmin);
	  //
	  for (int i=1; i<=n; ++i)
		s += c2 * wi[n][i] * func(c1 + c2 * xi[n][i]);
	}
  //
  return s;
}
//
// get the nodes of the Gauss-Legendre rule (the zeros of Pn(x))
// by the Newton method
//
void gauss_legendre_rule::get_node(void)
{
  for (int n=2; n<=nmax; ++n)
	{
	  for (int i=1; i<=n; ++i)
		{
		  double eps0 = 1.0e-15, eps1, eps2;
		  double x0, x1, dx, px, dpx;
		  //
		  // initial guess for the Newton method
		  //
		  x0 = cos(M_PI * ((double)i-0.25) / ((double)n+0.5));
		  //
		  // the Newton method
		  //
		  int l_conv = 0;
		  int kmax = 10;
		  for (int k=1; k<=kmax; ++k)
			{
			  dx = - pol(n, x0) / dpol(n, x0);
			  x1 = x0 + dx;
			  //
			  // if |(new x) - (old x)| < eps * |(new x)| or
			  //    |f(x)| < eps * |f'(x)|,
			  // stop the iteration of the Newton method
			  //
			  px = fabs(pol(n, x1));
			  dpx = fabs(dpol(n, x1));
			  eps1 = eps2 = eps0;
			  if (x1!=0.0) eps1 *= fabs(x1);
			  if (dpx!=0.0) eps2 *= dpx;
			  if (fabs(dx) < eps1 || px < eps2) {
				++l_conv;
				break;
			  }
			  x0 = x1;
			}
		  if (l_conv==0)
			cout << "Newton method not convergent: "
				 << "n = " << setw(3) << n << " i = " << setw(3) << i << endl;
		  xi[n][i] = x1;
		}
	}
}
//
// get the weights of the Gauss-Legendre rule
//
void gauss_legendre_rule::get_weight(void)
{
  for (int n=2; n<=nmax; ++n)
	for (int i=1; i<=n; ++i)
	  {
		double x, np;
		x = xi[n][i];
		np = (double)n * pol(n-1, x);
		wi[n][i] = 2.0 * (1.0 - x*x) / np / np;
	  }
}
//
// the Legendre polynomial of order n : Pn(x)
//
double gauss_legendre_rule::pol(int n, double x)
{
  double p;
  //
  if (n<0) cout << "n must satisfy n>=0" << endl;
  //
  if (n==0) p = 1.0;
  if (n==1) p = x;
  if (n>=2) {
	double p_2, p_1;
	p_1 = 1.0;
	p = x;
	for (int k=2; k<=n; ++k)
	  {
		p_2 = p_1;
		p_1 = p;
		p = ((double)(2*k-1)*x*p_1 - (double)(k-1)*p_2) / (double)k;
	  }
  }
  return p;
}
//
// the derivative of the Legendre polynomial of order n : Pn'(x)
//
double gauss_legendre_rule::dpol(int n, double x)
{
  double dp;
  //
  if (n==0)
	dp = 0.0;
  else 
	dp = - (double)n * (x * pol(n,x) - pol(n-1,x)) / (1.0 - x*x);
  //
  return dp;
}
//
//---------------------------------------------------------------
//
double func1(double x)
{
  return 1.0 / (1.0 + x*x);
}
//
double func2(double x)
{
  return exp(-x);
}
//
int main(void)
{
  cout << setiosflags(ios::scientific);
  //
  cout << "# " << endl;
  cout << "# numerical integration of f(x) over (xmin, xmax)" << endl;
  cout << "# by the Gauss-Legendre rule" << endl;
  cout << "# " << endl;
  //
  int nmax;
  //
  nmax = 12;
  gauss_legendre_rule gauss_legendre_rule1(nmax);
  //
  double xmin, xmax, s, se, error;
  xmin = 0.0;
  xmax = 1.0;
  se = 0.25 * M_PI;
  cout << "# " << endl;
  cout << "# integral 1 : f(x) = 1/(1+x^2), xmin = " << setprecision(5) << xmin
	   << ", xmax = " << setprecision(5) << xmax << endl;
  cout << "# I(exact) = " << se << endl;
  cout << "# n      I(numerical)      relative error" << endl;
  cout << "#----------------------------------------" << endl;
  for (int n=2; n<=nmax; ++n)
	{
	  s = gauss_legendre_rule1.integral(func1, xmin, xmax, n);
	  error = fabs(s - se) / fabs(se);
	  cout << setw(3) << n << " " << setprecision(15) << s << " "
		   << setprecision(1) << error << endl;
	}
  //
  xmin = 1.0;
  xmax = 2.0;
  se = exp(-1.0) - exp(-2.0);
  cout << "# " << endl;
  cout << "# integral 2 : f(x) = exp(-x), xmin = " << setprecision(5) << xmin
	   << ", xmax = " << setprecision(5) << xmax << endl;
  cout << "# I(exact) = " << se << endl;
  cout << "# n      I(numerical)      relative error" << endl;
  cout << "#----------------------------------------" << endl;
  for (int n=2; n<=nmax; ++n)
	{
	  s = gauss_legendre_rule1.integral(func2, xmin, xmax, n);
	  error = fabs(s - se) / fabs(se);
	  cout << setw(3) << n << " " << setprecision(15) << s << " "
		   << setprecision(1) << error << endl;
	}
  //
  return 0;
}
